import { PushDetail, PushResult } from '../../../typings';
import { TaskParser } from '../types';
export declare const parsePushResult: TaskParser<string, PushResult>;
export declare const parsePushDetail: TaskParser<string, PushDetail>;
