import { InitResult } from '../../../typings';
import { StringTask } from '../types';
export declare function initTask(bare: boolean | undefined, path: string, customArgs: string[]): StringTask<InitResult>;
