import type { SimpleGitPlugin } from './simple-git-plugin';
import type { SimpleGitPluginConfig } from '../types';
export declare function isCloneUploadPackSwitch(char: string, arg: string | unknown): boolean;
export declare function blockUnsafeOperationsPlugin({ allowUnsafePack, ...options }?: SimpleGitPluginConfig['unsafe']): SimpleGitPlugin<'spawn.args'>;
