/**
Returns whether a screen reader is enabled. This is useful when you want to render different output for screen readers.
*/
declare const useIsScreenReaderEnabled: () => boolean;
export default useIsScreenReaderEnabled;
