import type { Input, Options } from '../types/options.js';
import { type ResponsePromise } from '../types/ResponsePromise.js';
export declare class Ky {
    #private;
    static create(input: Input, options: Options): ResponsePromise;
    request: Request;
    constructor(input: Input, options?: Options);
}
