/**
 * Known process exit codes used by the task parsers to determine whether an error
 * was one they can automatically handle
 */
export declare enum ExitCodes {
    SUCCESS = 0,
    ERROR = 1,
    NOT_FOUND = -2,
    UNCLEAN = 128
}
