export declare const fetch: typeof globalThis.fetch;
export declare const Blob: typeof globalThis.Blob;
export declare const File: typeof globalThis.File;
export declare const FormData: typeof globalThis.FormData;
export declare const Headers: typeof globalThis.Headers;
export declare const Request: typeof globalThis.Request;
export declare const Response: typeof globalThis.Response;
export declare const AbortController: typeof globalThis.AbortController;

export default fetch;
