import createReconciler from 'react-reconciler';
import { type TextNode, type DOMElement } from './dom.js';
declare const _default: createReconciler.Reconciler<DOMElement, DOMElement, TextNode, DOMElement, unknown, unknown>;
export default _default;
