import type { Options } from '../types/options.js';
export declare const getBodySize: (body?: BodyInit | null) => number;
export declare const streamResponse: (response: Response, onDownloadProgress: Options["onDownloadProgress"]) => Response;
export declare const streamRequest: (request: Request, onUploadProgress: Options["onUploadProgress"], originalBody?: BodyInit | null) => Request;
